<?php
namespace Blesta\Core\Pricing\Presenter\Items;

use Blesta\Core\Pricing\Presenter\Items\Service\ServiceItems;
use Blesta\Core\Pricing\Presenter\Items\Service\ServiceDataItems;
use Blesta\Core\Pricing\PricingFactory;
use Blesta\Items\Item\ItemInterface;
use Blesta\Items\ItemFactory;
use Blesta\Items\Collection\ItemCollection;

/**
 * Instantiates service item objects
 */
class ServiceFactory
{
    /**
     * Creates an instance of ServiceItems
     *
     * @param Blesta\Core\Pricing\PricingFactory $pricingFactory An instance of the PricingFactory
     * @param Blesta\Items\ItemFactory $itemFactory An instance of the ItemFactory
     * @param Blesta\Items\Item\ItemInterface $settings An item containing a set of settings
     * @param Blesta\Items\Collection\ItemCollection $taxes A collection of items representing taxes
     * @param Blesta\Items\Collection\ItemCollection $discounts A collection of items representing discounts
     * @param Blesta\Items\Item\ItemInterface $options An item containing a set of custom options
     */
    public function service(
        PricingFactory $pricingFactory,
        ItemFactory $itemFactory,
        ItemInterface $settings,
        ItemCollection $taxes = null,
        ItemCollection $discounts = null,
        ItemInterface $options = null
    ) {
        return new ServiceItems($pricingFactory, $itemFactory, $settings, $taxes, $discounts, $options);
    }

    /**
     * Creates an instance of ServiceDataItems
     *
     * @param Blesta\Core\Pricing\PricingFactory $pricingFactory An instance of the PricingFactory
     * @param Blesta\Items\ItemFactory $itemFactory An instance of the ItemFactory
     * @param Blesta\Items\Item\ItemInterface $settings An item containing a set of settings
     * @param Blesta\Items\Collection\ItemCollection $taxes A collection of items representing taxes
     * @param Blesta\Items\Collection\ItemCollection $discounts A collection of items representing discounts
     * @param Blesta\Items\Item\ItemInterface $options An item containing a set of custom options
     */
    public function serviceData(
        PricingFactory $pricingFactory,
        ItemFactory $itemFactory,
        ItemInterface $settings,
        ItemCollection $taxes = null,
        ItemCollection $discounts = null,
        ItemInterface $options = null
    ) {
        return new ServiceDataItems($pricingFactory, $itemFactory, $settings, $taxes, $discounts, $options);
    }
}
